<?php
/**************************************************************************
*                        GONICUS Server Administration                    *
*        Version 1.0.0  (C) 2002 Cajus Pollmeier <pollmeier@gonicus.de>   *
***************************************************************************
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation, version 2 of the License                *
*                                                                         *
***************************************************************************/

function ldap_get_list ($pattern){
	global $LDAP_SERVER, $LDAP_PORT, $LDAP_BASE;
	
	# Connect to LDAP-Server
        if (!($ldap = ldap_connect($LDAP_SERVER, $LDAP_PORT))) {
		print_red (_("Can't connect to the LDAP server").
			" ($LDAP_SERVER:$LDAP_PORT): ".ldap_error ($ldap));
		exit ();
	}
	
	# do an anonymous bind to get user list
	if (!($ldap_handle = @ldap_bind($ldap))) {
		print_red (_("Can't authenticate to the LDAP server: ").
			ldap_error ($ldap));
		exit ();
	}

	# build complete array, or user name array?
	$sr=ldap_search ($ldap, $LDAP_BASE, "$pattern");
	$info = ldap_get_entries($ldap, $sr);

	ldap_close($ldap);
	return ($info);
}


function ldap_get_user_list ($user=""){
	if (strlen("$user")){
		return ldap_get_list ("uid=$user");
	}
	else{
		return ldap_get_list ("uid=*");
	}
}


function ldap_get_group_list ($gid=""){
        # build complete array, or user name array?
        if (strlen("$gid")){
		return ldap_get_list ("(&(gidNumber=$gid)(objectClass=posixGroup))");
	}
	else{
		return ldap_get_list ("(&(gidNumber=*)(objectClass=posixGroup))");
	}
}


function ldap_check_auth ($user="", $password){
	if (!strlen($user)){
		return false;
	}
	if (!($ldap=ldap_con ($user, $password))){
		return false;
	}
	ldap_close ($ldap);

	return true;
}


function ldap_con ($user="", $pass=""){
	global $LDAP_SERVER, $LDAP_PORT, $LDAP_BASE;

	if (!($ldap= ldap_connect($LDAP_SERVER, $LDAP_PORT))) {
		print_red (_("Can't connect to the LDAP server").
                        " ($LDAP_SERVER:$LDAP_PORT): ".ldap_error ($ldap));

		return false;
	}

	# Anmeldung vornehmen
	if ($user==""){
		if (!ldap_bind($ldap)){
			print_red (_("Can't do an anonymous bind to the LDAP server."));
			ldap_close ($ldap);

			return false;
		}
	}
	else{
	        if (!ldap_bind($ldap, $user, $pass)){
			print_red (_("Can't bind to LDAP server: ").ldap_error($ldap));
	                ldap_close ($ldap);

	                return false;
	        }
	}
	return $ldap;
}


function ldap_change_password ($user, $pass, $newpass, $name=""){
	if (!($ldap=ldap_con ($user, $pass))) return false;

	# Passwort Hash erzeugen
	$info["userPassword"]= "{crypt}".crypt($newpass,substr($session_id,0,2));

	if (!$name){
		$name= $user;
	}

	# Eintrag anpassen
	if (!ldap_modify($ldap, $name, $info)){
		return false;
	}

	ldap_close($ldap);

	return true;
}

function ldap_is_member ($list,$type,$nr=0){
        for ($i=0; $i<$list[$nr]["objectclass"]["count"]; $i++){
                if ($list[$nr]["objectclass"][$i]==$type){
			return true;
		}
        }
        return false;
}


function ldap_change_smb_password ($user, $pass)
{
	# check for objectclass sambaAccount
	$me= ldap_get_user_list ($user);
	
	if (!ldap_is_member($me,"sambaAccount")){
		return true;
	}

	$dn= $me[0]["dn"];
	if (!($ldap=ldap_con ($dn, $pass))){
		return false;
	}

	# Passwort Hash erzeugen
	if (!($hash= `mkntpwd $pass`)){
		print_red (_("Can't execute the <b>mkntpwd</b> binary to create samba passwords!"));
	}
	$ntPassword= split(':',chop($hash));
        $info["lmpassword"]= $ntPassword[0];
        $info["ntpassword"]= $ntPassword[1];

	# Eintrag anpassen
        if (!ldap_modify($ldap, $dn, $info)){
		print_red (_("Can't change the samba password: ").
			ldap_error ($ldap)."<br>");
                return false;
	}

	ldap_close($ldap);
	return true;	
}


function get_entry($list,$name,$nr=0){
	$value="";

        if (is_array ($list[$nr]["$name"])){
                for ($i=0; $i<$list[$nr]["$name"]["count"]; $i++){
			$value=$value.$list[$nr]["$name"][$i];
		}
        }
        else{
                $value= $list[$nr]["$name"];
        }

        return $value;
}


function get_group_member_list($list,$name){
        $value= array();

        for ($i=0; $i<$list["count"]; $i++){
                for ($j=0;$j<$list[$i]["memberuid"]["count"];$j++){
                        if ($list[$i]["memberuid"][$j] == $name){
                                $value[]=$list[$i]["cn"][0];
                        }
                }
        }

        return $value;
}

function get_next_entry ($list, $type){
        global $ID_BASE;
        $idlist= array();

        for ($i=0; $i<$list["count"]; $i++){
                $idlist[$i]= get_entry($list,$type,$i);
        }
        sort ($idlist, SORT_NUMERIC);
        $searchid= $ID_BASE;

	# look for space
        for ($i=0; $i<count($idlist); $i++){
                $currentid= $idlist[$i];
                if ($currentid <= $searchid){
			continue;
		}
                $searchid++;
                if ($currentid!=$searchid){
			break;
		}
        }

	# no space found?
	if ($currentid == $searchid){
		$searchid++;
	}

	if ($searchid>=65000){
		print_red (_("No free ID's available!"));
		exit ();
	}

	return $searchid;
}

function get_next_uid ($list){
        return (get_next_entry ($list,"uidnumber"));
}


function get_next_gid ($list){
        return (get_next_entry ($list,"gidnumber"));
}


function get_group_dn ($group){
	global $LDAP_BASE;

	$ldap= ldap_con ();

	$filter="(&(objectclass=posixGroup)(cn=$group))";
	$sr=ldap_search ($ldap, $LDAP_BASE, $filter);
	$info = ldap_get_entries($ldap, $sr);

	return ($info[0]["dn"]);
}


function ldap_add_groupmember($ldap, $uid, $gid){
	$entry= array();
	$entry["memberuid"] = $uid;
	$groupdn= get_group_dn ($gid);

	if (!ldap_mod_add($ldap, $groupdn, $entry)){
		return false;
	}
	return true;
}


function ldap_del_groupmember($ldap, $uid, $gid){
	$entry= array();
	$entry["memberuid"] = $uid;
	$groupdn= get_group_dn ($gid);

	if (!ldap_mod_del($ldap, $groupdn, $entry)){
		return false;
	}
	return true;
}


?>
